use strict;
use warnings FATAL => 'all';

use Apache::Test;
use Apache::TestRequest;

plan tests => 1, need_php;

my $expected = <<EXPECT;
&lt;&gt;&quot;&amp;εΔ
&lt;&gt;&quot;&amp;&aring;&Auml;
EXPECT

my $result = GET_BODY "/php/strings4.php";

ok $result eq $expected;
