use strict;
use warnings FATAL => 'all';

use Apache::Test;
use Apache::TestRequest;

plan tests => 1, need_php;

my $expected = "\"	\\'\\n\\'a\\\\b\\";

my $result = GET_BODY "/php/strings.php";
ok $result eq $expected;
